@@ -28,7 +28,7 @@ impl UpdateCache for InteractionCreate {
                 }
 
                 if let Some(resolved) = &command.data.resolved {
-                    for u in &resolved.users {
+                    for u in resolved.users.values() {
                         cache.cache_user(Cow::Borrowed(u), command.guild_id);
 
                         if !cache.wants(ResourceType::MEMBER) || command.guild_id.is_none() {
@@ -37,16 +37,21 @@ impl UpdateCache for InteractionCreate {
 
                         // This should always match, because resolved members
                         // are guaranteed to have a matching resolved user
-                        if let Some(member) = &resolved.members.iter().find(|m| m.id == u.id) {
+                        if let Some((&id, member)) =
+                            &resolved.members.iter().find(|(&id, _)| id == u.id)
+                        {
                             if let Some(guild_id) = command.guild_id {
-                                cache.cache_borrowed_interaction_member(guild_id, member);
+                                cache.cache_borrowed_interaction_member(guild_id, member, id);
                             }
                         }
                     }
 
                     if cache.wants(ResourceType::ROLE) {
                         if let Some(guild_id) = command.guild_id {
-                            cache.cache_roles(guild_id, resolved.roles.iter().cloned());
+                            cache.cache_roles(
+                                guild_id,
+                                resolved.roles.iter().map(|(_, v)| v).cloned(),
+                            );
                         }
                     }
                 }
@@ -59,6 +64,7 @@ impl UpdateCache for InteractionCreate {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use std::collections::HashMap;
     use twilight_model::{
         application::interaction::{
             application_command::{CommandData, CommandInteractionDataResolved, InteractionMember},
@@ -93,106 +99,120 @@ mod tests {
                     name: "command name".into(),
                     options: Vec::new(),
                     resolved: Some(CommandInteractionDataResolved {
-                        channels: Vec::new(),
-                        members: Vec::from([InteractionMember {
-                            hoisted_role: None,
-                            id: UserId::new(7).expect("non zero"),
-                            joined_at: timestamp,
-                            nick: None,
-                            premium_since: None,
-                            roles: vec![RoleId::new(8).expect("non zero")],
-                        }]),
-                        messages: Vec::from([Message {
-                            activity: None,
-                            application: None,
-                            application_id: None,
-                            attachments: Vec::new(),
-                            author: User {
+                        channels: HashMap::new(),
+                        members: IntoIterator::into_iter([(
+                            UserId::new(7).expect("non zero"),
+                            InteractionMember {
+                                joined_at: timestamp,
+                                nick: None,
+                                premium_since: None,
+                                roles: vec![RoleId::new(8).expect("non zero")],
+                            },
+                        )])
+                        .collect(),
+                        messages: IntoIterator::into_iter([(
+                            MessageId::new(4).expect("non zero"),
+                            Message {
+                                activity: None,
+                                application: None,
+                                application_id: None,
+                                attachments: Vec::new(),
+                                author: User {
+                                    accent_color: None,
+                                    avatar: Some("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa".to_owned()),
+                                    banner: None,
+                                    bot: false,
+                                    discriminator: 1,
+                                    email: None,
+                                    flags: None,
+                                    id: UserId::new(3).expect("non zero"),
+                                    locale: None,
+                                    mfa_enabled: None,
+                                    name: "test".to_owned(),
+                                    premium_type: None,
+                                    public_flags: None,
+                                    system: None,
+                                    verified: None,
+                                },
+                                channel_id: ChannelId::new(2).expect("non zero"),
+                                components: Vec::new(),
+                                content: "ping".to_owned(),
+                                edited_timestamp: None,
+                                embeds: Vec::new(),
+                                flags: Some(MessageFlags::empty()),
+                                guild_id: Some(GuildId::new(1).expect("non zero")),
+                                id: MessageId::new(4).expect("non zero"),
+                                interaction: None,
+                                kind: MessageType::Regular,
+                                member: Some(PartialMember {
+                                    avatar: None,
+                                    deaf: false,
+                                    joined_at: timestamp,
+                                    mute: false,
+                                    nick: Some("member nick".to_owned()),
+                                    permissions: None,
+                                    premium_since: None,
+                                    roles: Vec::new(),
+                                    user: None,
+                                }),
+                                mention_channels: Vec::new(),
+                                mention_everyone: false,
+                                mention_roles: Vec::new(),
+                                mentions: Vec::new(),
+                                pinned: false,
+                                reactions: Vec::new(),
+                                reference: None,
+                                sticker_items: vec![MessageSticker {
+                                    format_type: StickerFormatType::Png,
+                                    id: StickerId::new(1).expect("non zero"),
+                                    name: "sticker name".to_owned(),
+                                }],
+                                referenced_message: None,
+                                thread: None,
+                                timestamp,
+                                tts: false,
+                                webhook_id: None,
+                            },
+                        )])
+                        .collect(),
+                        roles: IntoIterator::into_iter([(
+                            RoleId::new(8).expect("non zero"),
+                            Role {
+                                color: 0u32,
+                                hoist: false,
+                                icon: None,
+                                id: RoleId::new(8).expect("non zero"),
+                                managed: false,
+                                mentionable: true,
+                                name: "role name".into(),
+                                permissions: Permissions::empty(),
+                                position: 2i64,
+                                tags: None,
+                                unicode_emoji: None,
+                            },
+                        )])
+                        .collect(),
+                        users: IntoIterator::into_iter([(
+                            UserId::new(7).expect("non zero"),
+                            User {
                                 accent_color: None,
-                                avatar: Some("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa".to_owned()),
+                                avatar: Some("different avatar".into()),
                                 banner: None,
                                 bot: false,
-                                discriminator: 1,
+                                discriminator: 5678,
                                 email: None,
                                 flags: None,
-                                id: UserId::new(3).expect("non zero"),
+                                id: UserId::new(7).expect("non zero"),
                                 locale: None,
                                 mfa_enabled: None,
-                                name: "test".to_owned(),
+                                name: "different name".into(),
                                 premium_type: None,
                                 public_flags: None,
                                 system: None,
                                 verified: None,
                             },
-                            channel_id: ChannelId::new(2).expect("non zero"),
-                            components: Vec::new(),
-                            content: "ping".to_owned(),
-                            edited_timestamp: None,
-                            embeds: Vec::new(),
-                            flags: Some(MessageFlags::empty()),
-                            guild_id: Some(GuildId::new(1).expect("non zero")),
-                            id: MessageId::new(4).expect("non zero"),
-                            interaction: None,
-                            kind: MessageType::Regular,
-                            member: Some(PartialMember {
-                                avatar: None,
-                                deaf: false,
-                                joined_at: timestamp,
-                                mute: false,
-                                nick: Some("member nick".to_owned()),
-                                permissions: None,
-                                premium_since: None,
-                                roles: Vec::new(),
-                                user: None,
-                            }),
-                            mention_channels: Vec::new(),
-                            mention_everyone: false,
-                            mention_roles: Vec::new(),
-                            mentions: Vec::new(),
-                            pinned: false,
-                            reactions: Vec::new(),
-                            reference: None,
-                            sticker_items: vec![MessageSticker {
-                                format_type: StickerFormatType::Png,
-                                id: StickerId::new(1).expect("non zero"),
-                                name: "sticker name".to_owned(),
-                            }],
-                            referenced_message: None,
-                            thread: None,
-                            timestamp,
-                            tts: false,
-                            webhook_id: None,
-                        }]),
-                        roles: Vec::from([Role {
-                            color: 0u32,
-                            hoist: false,
-                            icon: None,
-                            id: RoleId::new(8).expect("non zero"),
-                            managed: false,
-                            mentionable: true,
-                            name: "role name".into(),
-                            permissions: Permissions::empty(),
-                            position: 2i64,
-                            tags: None,
-                            unicode_emoji: None,
-                        }]),
-                        users: Vec::from([User {
-                            accent_color: None,
-                            avatar: Some("different avatar".into()),
-                            banner: None,
-                            bot: false,
-                            discriminator: 5678,
-                            email: None,
-                            flags: None,
-                            id: UserId::new(7).expect("non zero"),
-                            locale: None,
-                            mfa_enabled: None,
-                            name: "different name".into(),
-                            premium_type: None,
-                            public_flags: None,
-                            system: None,
-                            verified: None,
-                        }]),
+                        )])
+                        .collect(),
                     }),
                 },
                 guild_id: Some(GuildId::new(3).expect("non zero")),
